/*
  calendricalmodule.c: Copyright (C) 1997-2002,2007,2014,2018 Tadayoshi Funaba All rights reserved
  $Id: calendricalmodule.c,v 1.17 2018/07/05 10:14:22 tadf Exp $
*/

#include "ruby.h"
#ifdef HAVE_RUBY_ENCODING_H
#include "ruby/encoding.h"
#endif
#include "calendrical.h"
#include "calendrical2.h"
#include "qref.h"

#define INT2BOOL(x) ((x) ? Qtrue : Qfalse)
#define BOOL2INT(x) (RTEST(x) ? 1 : 0)

#define CHECK_RANGE(v, m)			\
{						\
    if ((v) < (m))				\
	rb_raise(rb_eArgError, "out of range");	\
}

#define CHECK_DOMAIN(v, m)			\
{						\
    if ((v) < (m))				\
	rb_raise(rb_eArgError, "out of domain");\
}

#define CHECK_DOMAIN2(v, m, n)			\
{						\
    if ((v) < (m) || (v) > (n))			\
	rb_raise(rb_eArgError, "out of domain");\
}

static VALUE
calendrical_gregorian_leap_year(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = gregorian_leap_year(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_gregorian_last_day_of_month(VALUE obj, VALUE y, VALUE m)
{
    int iy, im, md;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    iy = NUM2INT(y);
    md = gregorian_last_day_of_month(iy, im);
    return INT2NUM(md);
}

static VALUE
calendrical_gregorian_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 31);
    iy = NUM2INT(y);
    a = gregorian_to_rd(iy, im, imd);
    rd_to_gregorian(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_gregorian(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_gregorian(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_kday_on_or_before(VALUE obj, VALUE a, VALUE k)
{
    int ia, ik, b;

    ia = NUM2INT(a);
    ik = NUM2INT(k); CHECK_DOMAIN2(ik, 0, 6);
    b = kday_on_or_before(ia, NUM2INT(k));
    return INT2NUM(b);
}

static VALUE
calendrical_iso_to_rd(VALUE obj, VALUE y, VALUE w, VALUE wd)
{
    int iy, iw, iwd, a, iy2, iw2, iwd2;

    iw = NUM2INT(w); CHECK_DOMAIN2(iw, 1, 53);
    iwd = NUM2INT(wd); CHECK_DOMAIN2(iwd, 1, 7);
    iy = NUM2INT(y);
    a = iso_to_rd(iy, iw, iwd);
    rd_to_iso(a, &iy2, &iw2, &iwd2);
    if (iw != iw2 || iwd != iwd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_iso(VALUE obj, VALUE a)
{
    int ia, y, w, wd;

    ia = NUM2INT(a);
    rd_to_iso(ia, &y, &w, &wd);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(w), INT2NUM(wd));
}

static VALUE
calendrical_julian_leap_year(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = julian_leap_year(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_julian_last_day_of_month(VALUE obj, VALUE y, VALUE m)
{
    int iy, im, md;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    iy = NUM2INT(y);
    md = julian_last_day_of_month(iy, im);
    return INT2NUM(md);
}

static VALUE
calendrical_julian_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 31);
    iy = NUM2INT(y);
    a = julian_to_rd(iy, im, imd);
    rd_to_julian(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_julian(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_julian(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_islamic_leap_year(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = islamic_leap_year(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_islamic_last_day_of_month(VALUE obj, VALUE y, VALUE m)
{
    int iy, im, md;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    iy = NUM2INT(y);
    md = islamic_last_day_of_month(iy, im);
    return INT2NUM(md);
}

static VALUE
calendrical_islamic_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 30);
    iy = NUM2INT(y);
    a = islamic_to_rd(iy, im, imd);
    rd_to_islamic(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_islamic(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_islamic(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_hebrew_leap_year(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = hebrew_leap_year(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_hebrew_last_month_of_year(VALUE obj, VALUE y)
{
    int iy, m;

    iy = NUM2INT(y);
    m = hebrew_last_month_of_year(iy);
    return INT2NUM(m);
}

static VALUE
calendrical_hebrew_last_day_of_month(VALUE obj, VALUE y, VALUE m)
{
    int iy, im, md;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 13);
    iy = NUM2INT(y);
    md = hebrew_last_day_of_month(iy, im);
    return INT2NUM(md);
}

static VALUE
calendrical_hebrew_calendar_elapsed_days(VALUE obj, VALUE y)
{
    int iy, d;

    iy = NUM2INT(y);
    d = hebrew_calendar_elapsed_days(iy);
    return INT2NUM(d);
}

static VALUE
calendrical_hebrew_days_in_year(VALUE obj, VALUE y)
{
    int iy, d;

    iy = NUM2INT(y);
    d = hebrew_days_in_year(iy);
    return INT2NUM(d);
}

static VALUE
calendrical_long_heshvan(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = long_heshvan(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_short_kislev(VALUE obj, VALUE y)
{
    int iy, s;

    iy = NUM2INT(y);
    s = short_kislev(iy);
    return INT2BOOL(s);
}

static VALUE
calendrical_hebrew_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 13);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 30);
    iy = NUM2INT(y);
    a = hebrew_to_rd(iy, im, imd);
    rd_to_hebrew(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_hebrew(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_hebrew(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_nth_kday(VALUE obj, VALUE y, VALUE m, VALUE n, VALUE k)
{
    int iy, im, in, ik, b;

    in = NUM2INT(n);
    if (in < -5 || in == 0 || in > 5)
	rb_raise(rb_eArgError, "out of range");
    ik = NUM2INT(k); CHECK_DOMAIN2(ik, 0, 6);
    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    iy = NUM2INT(y);
    b = nth_kday(iy, im, in, ik);
    return INT2NUM(b);
}

static VALUE
calendrical_nicaean_rule_easter(VALUE obj, VALUE y)
{
    int iy, a;

    iy = NUM2INT(y);
    a = nicaean_rule_easter(iy);
    return INT2NUM(a);
}

static VALUE
calendrical_easter(VALUE obj, VALUE y)
{
    int iy, a;

    iy = NUM2INT(y);
    a = easter(iy);
    return INT2NUM(a);
}

static VALUE
calendrical_mayan_long_count_to_rd
(VALUE obj, VALUE bt, VALUE kt, VALUE t, VALUE u, VALUE k)
{
    int ibt, ikt, it, iu, ik, a, ibt2, ikt2, it2, iu2, ik2;

    ibt = NUM2INT(bt);
    ikt = NUM2INT(kt); CHECK_DOMAIN2(ikt, 0, 19);
    it = NUM2INT(t); CHECK_DOMAIN2(it, 0, 19);
    iu = NUM2INT(u); CHECK_DOMAIN2(iu, 0, 17);
    ik = NUM2INT(k); CHECK_DOMAIN2(ik, 0, 19);
    a = mayan_long_count_to_rd(ibt, ikt, it, iu, ik);
    rd_to_mayan_long_count(a, &ibt2, &ikt2, &it2, &iu2, &ik2);
    if (ibt != ibt2 || ikt != ikt2 || it != it2 || iu != iu2 || ik != ik2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_mayan_long_count(VALUE obj, VALUE a)
{
    int ia, bt, kt, t, u, k;

    ia = NUM2INT(a);
    rd_to_mayan_long_count(ia, &bt, &kt, &t, &u, &k);
    return rb_ary_new3
	(5, INT2NUM(bt), INT2NUM(kt), INT2NUM(t), INT2NUM(u), INT2NUM(k));
}

static VALUE
calendrical_rd_to_mayan_haab(VALUE obj, VALUE a)
{
    int ia, md, m;

    ia = NUM2INT(a);
    rd_to_mayan_haab(ia, &m, &md);
    return rb_ary_new3(2, INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_mayan_haab_difference
(VALUE obj, VALUE m1, VALUE md1, VALUE m2, VALUE md2)
{
    int imd1, im1, imd2, im2, d;

    im1 = NUM2INT(m1); CHECK_DOMAIN2(im1, 1, 19);
    imd1 = NUM2INT(md1); CHECK_DOMAIN2(imd1, 0, (im1 == 19) ? 4 : 19);
    im2 = NUM2INT(m2); CHECK_DOMAIN2(im2, 1, 19);
    imd2 = NUM2INT(md2); CHECK_DOMAIN2(imd2, 0, (im2 == 19) ? 4 : 19);
    d = mayan_haab_difference(im1, imd1, im2, imd2);
    return INT2NUM(d);
}

static VALUE
calendrical_mayan_haab_on_or_before(VALUE obj, VALUE m, VALUE md, VALUE a)
{
    int imd, im, ia, d;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 19);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 0, (im == 19) ? 4 : 19);
    ia = NUM2INT(a);
    d = mayan_haab_on_or_before(im, imd, ia);
    return INT2NUM(d);
}

static VALUE
calendrical_rd_to_mayan_tzolkin(VALUE obj, VALUE a)
{
    int ia, nu, na;

    ia = NUM2INT(a);
    rd_to_mayan_tzolkin(ia, &na, &nu);
    return rb_ary_new3(2, INT2NUM(na), INT2NUM(nu));
}

static VALUE
calendrical_mayan_tzolkin_difference
(VALUE obj, VALUE na1, VALUE nu1, VALUE na2, VALUE nu2)
{
    int inu1, ina1, inu2, ina2, d;

    inu1 = NUM2INT(nu1); CHECK_DOMAIN2(inu1, 1, 13);
    ina1 = NUM2INT(na1); CHECK_DOMAIN2(ina1, 1, 20);
    inu2 = NUM2INT(nu2); CHECK_DOMAIN2(inu2, 1, 13);
    ina2 = NUM2INT(na2); CHECK_DOMAIN2(ina2, 1, 20);
    d = mayan_tzolkin_difference(ina1, inu1, ina2, inu2);
    return INT2NUM(d);
}

static VALUE
calendrical_mayan_tzolkin_on_or_before(VALUE obj, VALUE na, VALUE nu, VALUE a)
{
    int inu, ina, ia, d;

    inu = NUM2INT(nu); CHECK_DOMAIN2(inu, 1, 13);
    ina = NUM2INT(na); CHECK_DOMAIN2(ina, 1, 20);
    ia = NUM2INT(a);
    d = mayan_tzolkin_on_or_before(ina, inu, ia);
    return INT2NUM(d);
}

static VALUE
calendrical_mayan_haab_tzolkin_on_or_before
(VALUE obj, VALUE m, VALUE md, VALUE na, VALUE nu, VALUE a)
{
    int imd, im, inu, ina, ia, d;

    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 0, 19);
    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 19);
    inu = NUM2INT(nu); CHECK_DOMAIN2(inu, 1, 13);
    ina = NUM2INT(na); CHECK_DOMAIN2(ina, 1, 20);
    ia = NUM2INT(a);
    d = mayan_haab_tzolkin_on_or_before(im, imd, ina, inu, ia);
    return INT2NUM(d);
}

static VALUE
calendrical_french_leap_year(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = french_leap_year(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_french_last_day_of_month(VALUE obj, VALUE y, VALUE m)
{
    int iy, im, md;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 13);
    iy = NUM2INT(y);
    md = french_last_day_of_month(iy, im);
    return INT2NUM(md);
}

static VALUE
calendrical_french_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 13);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 30);
    iy = NUM2INT(y);
    a = french_to_rd(iy, im, imd);
    rd_to_french(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_french(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_french(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static double
getfloat(VALUE x)
{
    if (TYPE(x) == T_FLOAT)
#ifndef RFLOAT_VALUE
	return RFLOAT(x)->value;
#else
	return RFLOAT_VALUE(x);
#endif
    return (double)NUM2INT(x);
}

static VALUE
calendrical_solar_longitude(VALUE obj, VALUE d)
{
    double l;

    l = solar_longitude(getfloat(d));
    return rb_float_new(l);
}

static VALUE
calendrical_zodiac(VALUE obj, VALUE d)
{
    double l;

    l = zodiac(getfloat(d));
    return rb_float_new(l);
}

static VALUE
calendrical_rd_to_old_hindu_solar(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_old_hindu_solar(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_old_hindu_solar_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 31);
    iy = NUM2INT(y);
    a = old_hindu_solar_to_rd(iy, im, imd);
    rd_to_old_hindu_solar(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_lunar_longitude(VALUE obj, VALUE d)
{
    double l;

    l = lunar_longitude(getfloat(d));
    return rb_float_new(l);
}

static VALUE
calendrical_lunar_phase(VALUE obj, VALUE d)
{
    double p;

    p = lunar_phase(getfloat(d));
    return rb_float_new(p);
}

static VALUE
calendrical_new_moon(VALUE obj, VALUE d)
{
    double n;

    n = new_moon(getfloat(d));
    return rb_float_new(n);
}

static VALUE
calendrical_rd_to_old_hindu_lunar(VALUE obj, VALUE a)
{
    int ia, y, m, l, md;

    ia = NUM2INT(a);
    rd_to_old_hindu_lunar(ia, &y, &m, &l, &md);
    return rb_ary_new3(4, INT2NUM(y), INT2NUM(m), INT2BOOL(l), INT2NUM(md));
}

static VALUE
calendrical_old_hindu_lunar_precedes
(VALUE obj,
 VALUE y1, VALUE m1, VALUE l1, VALUE md1,
 VALUE y2, VALUE m2, VALUE l2, VALUE md2)
{
    int iy1, im1, il1, imd1, iy2, im2, il2, imd2, d;

    im1 = NUM2INT(m1); CHECK_DOMAIN2(im1, 1, 12);
    il1 = BOOL2INT(l1);
    imd1 = NUM2INT(md1); CHECK_DOMAIN2(imd1, 1, 30);
    iy1 = NUM2INT(y1);
    im2 = NUM2INT(m2); CHECK_DOMAIN2(im2, 1, 12);
    il2 = BOOL2INT(l2);
    imd2 = NUM2INT(md2); CHECK_DOMAIN2(imd2, 1, 30);
    iy2 = NUM2INT(y2);
    d = old_hindu_lunar_precedes(iy1, im1, il1, imd1, iy2, im2, il2, imd2);
    return INT2BOOL(d);
}

static VALUE
calendrical_old_hindu_lunar_to_rd
(VALUE obj, VALUE y, VALUE m, VALUE l, VALUE md)
{
    int iy, im, il, imd, a, iy2, im2, il2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    il = BOOL2INT(l);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 30);
    iy = NUM2INT(y);
    a = old_hindu_lunar_to_rd(iy, im, il, imd);
    rd_to_old_hindu_lunar(a, &iy2, &im2, &il2, &imd2);
    if (im != im2 || il != il2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_world_leap_year(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = world_leap_year(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_world_last_day_of_month(VALUE obj, VALUE y, VALUE m)
{
    int iy, im, md;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    iy = NUM2INT(y);
    md = world_last_day_of_month(iy, im);
    return INT2NUM(md);
}

static VALUE
calendrical_world_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 31);
    iy = NUM2INT(y);
    a = world_to_rd(iy, im, imd);
    rd_to_world(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_world(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_world(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_rd_to_world_day_of_week(VALUE obj, VALUE a)
{
    int ia, w;

    ia = NUM2INT(a);
    w = rd_to_world_day_of_week(ia);
    return INT2NUM(w);
}

static VALUE
calendrical_coptic_leap_year(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = coptic_leap_year(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_coptic_last_day_of_month(VALUE obj, VALUE y, VALUE m)
{
    int iy, im, md;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 13);
    iy = NUM2INT(y);
    md = coptic_last_day_of_month(iy, im);
    return INT2NUM(md);
}

static VALUE
calendrical_coptic_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 13);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 30);
    iy = NUM2INT(y);
    a = coptic_to_rd(iy, im, imd);
    rd_to_coptic(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_coptic(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_coptic(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_ethiopian_leap_year(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = ethiopian_leap_year(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_ethiopian_last_day_of_month(VALUE obj, VALUE y, VALUE m)
{
    int iy, im, md;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 13);
    iy = NUM2INT(y);
    md = ethiopian_last_day_of_month(iy, im);
    return INT2NUM(md);
}

static VALUE
calendrical_ethiopian_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 13);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 30);
    iy = NUM2INT(y);
    a = ethiopian_to_rd(iy, im, imd);
    rd_to_ethiopian(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_ethiopian(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_ethiopian(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_jalali_leap_year(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = jalali_leap_year(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_jalali_last_day_of_month(VALUE obj, VALUE y, VALUE m)
{
    int iy, im, md;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    iy = NUM2INT(y);
    md = jalali_last_day_of_month(iy, im);
    return INT2NUM(md);
}

static VALUE
calendrical_jalali_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 31);
    iy = NUM2INT(y);
    a = jalali_to_rd(iy, im, imd);
    rd_to_jalali(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_jalali(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_jalali(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_bahai_leap_year(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = bahai_leap_year(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_bahai_last_day_of_month(VALUE obj, VALUE y, VALUE m)
{
    int iy, im, md;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 20);
    iy = NUM2INT(y);
    md = bahai_last_day_of_month(iy, im);
    return INT2NUM(md);
}

static VALUE
calendrical_bahai_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 20);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 19);
    iy = NUM2INT(y);
    a = bahai_to_rd(iy, im, imd);
    rd_to_bahai(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_bahai(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_bahai(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_bahai_year_to_vahid(VALUE obj, VALUE y)
{
    int iy, k, v, iy2;

    iy = NUM2INT(y);
    bahai_year_to_vahid(iy, &k, &v, &iy2);
    return rb_ary_new3(3, INT2NUM(k), INT2NUM(v), INT2NUM(iy2));
}

static VALUE
calendrical_bahai_vahid_to_year(VALUE obj, VALUE k, VALUE v, VALUE y)
{
    int ik, iv, iy, iy2;

    ik = NUM2INT(k);
    iv = NUM2INT(v); CHECK_DOMAIN2(iv, 1, 19);
    iy = NUM2INT(y); CHECK_DOMAIN2(iy, 1, 19);
    iy2 = bahai_vahid_to_year(ik, iv, iy);
    return INT2NUM(iy2);
}

static VALUE
calendrical_indian_national_leap_year(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = indian_national_leap_year(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_indian_national_last_day_of_month(VALUE obj, VALUE y, VALUE m)
{
    int iy, im, md;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    iy = NUM2INT(y);
    md = indian_national_last_day_of_month(iy, im);
    return INT2NUM(md);
}

static VALUE
calendrical_indian_national_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 31);
    iy = NUM2INT(y);
    a = indian_national_to_rd(iy, im, imd);
    rd_to_indian_national(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_indian_national(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_indian_national(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_bengali_leap_year(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = bengali_leap_year(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_bengali_last_day_of_month(VALUE obj, VALUE y, VALUE m)
{
    int iy, im, md;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    iy = NUM2INT(y);
    md = bengali_last_day_of_month(iy, im);
    return INT2NUM(md);
}

static VALUE
calendrical_bengali_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 31);
    iy = NUM2INT(y);
    a = bengali_to_rd(iy, im, imd);
    rd_to_bengali(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_bengali(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_bengali(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_nanakshahi_leap_year(VALUE obj, VALUE y)
{
    int iy, l;

    iy = NUM2INT(y);
    l = nanakshahi_leap_year(iy);
    return INT2BOOL(l);
}

static VALUE
calendrical_nanakshahi_last_day_of_month(VALUE obj, VALUE y, VALUE m)
{
    int iy, im, md;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    iy = NUM2INT(y);
    md = nanakshahi_last_day_of_month(iy, im);
    return INT2NUM(md);
}

static VALUE
calendrical_nanakshahi_to_rd(VALUE obj, VALUE y, VALUE m, VALUE md)
{
    int iy, im, imd, a, iy2, im2, imd2;

    im = NUM2INT(m); CHECK_DOMAIN2(im, 1, 12);
    imd = NUM2INT(md); CHECK_DOMAIN2(imd, 1, 31);
    iy = NUM2INT(y);
    a = nanakshahi_to_rd(iy, im, imd);
    rd_to_nanakshahi(a, &iy2, &im2, &imd2);
    if (im != im2 || imd != imd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_nanakshahi(VALUE obj, VALUE a)
{
    int ia, y, m, md;

    ia = NUM2INT(a);
    rd_to_nanakshahi(ia, &y, &m, &md);
    return rb_ary_new3(3, INT2NUM(y), INT2NUM(m), INT2NUM(md));
}

static VALUE
calendrical_ordinal_to_rd(VALUE obj, VALUE y, VALUE yd)
{
    int iy, iyd, a, iy2, iyd2;

    iyd = NUM2INT(yd); CHECK_DOMAIN2(iyd, 1, 366);
    iy = NUM2INT(y);
    a = ordinal_to_rd(iy, iyd);
    rd_to_ordinal(a, &iy2, &iyd2);
    if (iyd != iyd2 || iy != iy2)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_ordinal(VALUE obj, VALUE a)
{
    int ia, y, yd;

    ia = NUM2INT(a);
    rd_to_ordinal(ia, &y, &yd);
    return rb_ary_new3(2, INT2NUM(y), INT2NUM(yd));
}

static VALUE
calendrical_jd_to_rd(VALUE obj, VALUE j)
{
    int ij, a;

    ij = NUM2INT(j);
    a = jd_to_rd(ij);
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_jd(VALUE obj, VALUE a)
{
    int ia, j;

    ia = NUM2INT(a);
    j = rd_to_jd(ia);
    return INT2NUM(j);
}

static VALUE
calendrical_mjd_to_rd(VALUE obj, VALUE m)
{
    int im, a;

    im = NUM2INT(m);
    a = mjd_to_rd(im);
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_mjd(VALUE obj, VALUE a)
{
    int ia, m;

    ia = NUM2INT(a);
    m = rd_to_mjd(ia);
    return INT2NUM(m);
}

static VALUE
calendrical_mjd_to_jd(VALUE obj, VALUE m)
{
    int im, j;

    im = NUM2INT(m);
    j = mjd_to_jd(im);
    return INT2NUM(j);

}

static VALUE
calendrical_jd_to_mjd(VALUE obj, VALUE j)
{
    int ij, m;

    ij = NUM2INT(j);
    m = jd_to_mjd(ij);
    return INT2NUM(m);
}

static VALUE
calendrical_ld_to_rd(VALUE obj, VALUE l)
{
    int il, a;

    il = NUM2INT(l);
    a = ld_to_rd(il);
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_ld(VALUE obj, VALUE a)
{
    int ia, l;

    ia = NUM2INT(a);
    l = rd_to_ld(ia);
    return INT2NUM(l);
}

static VALUE
calendrical_rd_to_day_of_week(VALUE obj, VALUE a)
{
    int ia, w;

    ia = NUM2INT(a);
    w = rd_to_day_of_week(ia);
    return INT2NUM(w);
}

static VALUE
calendrical_kyureki_to_rd(VALUE obj, VALUE y, VALUE m, VALUE l, VALUE md)
{
    int j, a;
    QDATE q, q2;

    q.j = 0;
    q.y = NUM2INT(y);
    q.yd = 0;
    q.m = NUM2INT(m); CHECK_DOMAIN2(q.m, 1, 12);
    q.md = NUM2INT(md); CHECK_DOMAIN2(q.md, 1, 30);
    q.wd = 0;
    q.leap = BOOL2INT(l);
    j = rqref(&q);
    if (j == 0)
	rb_raise(rb_eArgError, "invalid date");
    a = jd_to_rd(j);
    qref(j, &q2);
    if (q.y != q2.y || q.m != q2.m || q.md != q2.md || q.leap != q2.leap)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(a);
}

static VALUE
calendrical_rd_to_kyureki(VALUE obj, VALUE a)
{
    int ia, j;
    QDATE q;

    ia = NUM2INT(a); CHECK_DOMAIN2(ia, 162193, 767009);
    j = rd_to_jd(ia);
    qref(j, &q);
    if (q.j == 0)
	rb_raise(rb_eArgError, "invalid date");
    return rb_ary_new3
	(4, INT2NUM(q.y), INT2NUM(q.m), INT2BOOL(q.leap), INT2NUM(q.md));
}

static VALUE
calendrical_rd_to_kyureki_day_of_week(VALUE obj, VALUE a)
{
    int ia, j;
    QDATE q;

    ia = NUM2INT(a);
    j = rd_to_jd(ia);
    qref(j, &q);
    if (q.j == 0)
	rb_raise(rb_eArgError, "invalid date");
    return INT2NUM(q.wd);
}

static VALUE
mk_ary_of_str(long len, const char *a[])
{
    VALUE o;
    long i;

    o = rb_ary_new2(len);
    for (i = 0; i < len; i++) {
	VALUE e;

	if (!a[i])
	    e = Qnil;
	else {
	    e = rb_str_new2(a[i]);
#ifdef HAVE_RUBY_ENCODING_H
	    rb_enc_associate(e, rb_utf8_encoding());
	    ENC_CODERANGE_CLEAR(e);
#endif
	    rb_obj_freeze(e);
	}
	rb_ary_push(o, e);
    }
    rb_obj_freeze(o);
    return o;
}

static const char *day_names[] = {
    "Sunday", "Monday", "Tuesday",
    "Wednesday", "Thursday",
    "Friday", "Saturday"
};

static const char *month_names[] = {
    "*", "January", "February", "March",
    "April", "May", "June", "July",
    "August", "September", "October",
    "November", "December"
};

static const char *islamic_day_names[] = {
    "al-Aḥad", "al-Ithnayn",
    "ath-Thalaathaaʼ", "al-Arba‘aa’",
    "al-Khamīs", "al-Jumu‘ah", "as-Sabt"
};

static const char *islamic_month_names[] = {
    "*", "Muḥarram", "Ṣafar",
    "Rabīʿ I", "Rabīʿ II",
    "Jumādā I", "Jumādā II",
    "Rajab", "Shaʿbān",
    "Ramaḍān", "Shawwāl",
    "Dhū al-Qaʿda", "Dhū al-Ḥijja"
};

static const char *hebrew_day_names[] = {
    "Yom Rishon", "Yom Sheni", "Yom Shlishi",
    "Yom Reviʻi", "Yom Chamishi",
    "Yom Shishi", "Yom Shabbat"
};

static const char *hebrew_common_year_month_names[] = {
    "*", "Nisan", "Iyyar",
    "Sivan", "Tammuz", "Av",
    "Elul", "Tishri", "Heshvan",
    "Kislev", "Teveth", "Shevat",
    "Adar", "*"
};

static const char *hebrew_leap_year_month_names[] = {
    "*", "Nisan", "Iyyar",
    "Sivan", "Tammuz", "Av",
    "Elul", "Tishri", "Heshvan",
    "Kislev", "Teveth", "Shevat",
    "Adar I", "Adar II"
};

static const char *mayan_haab_month_names[] = {
    "*", "Pop", "Wo'",
    "Sip", "Sotz'", "Sek",
    "Xul", "Yaxk'in'", "Mol",
    "Ch'en", "Yax", "Sak'",
    "Keh", "Mak", "K'ank'in",
    "Muwan'", "Pax", "K'ayab",
    "Kumk'u", "Wayeb'"
};

static const char *mayan_tzolkin_month_names[] = {
    "*", "Imix'", "Ik'",
    "Ak'b'al", "K'an", "Chikchan",
    "Kimi", "Manik'", "Lamat",
    "Muluk", "Ok", "Chuwen",
    "Eb'", "B'en", "Ix",
    "Men", "K'ib'", "Kab'an",
    "Etz'nab'", "Kawak", "Ajaw"
};

static const char *french_day_names[] = {
    "Primidi", "Doudi",
    "Tridi", "Quartidi",
    "Quintidi", "Sextidi",
    "Septidi", "Octidi",
    "Nonidi", "Décadi"
};

static const char *french_sansculottides_names[] = {
    "Vertu", "Génie",
    "Labour", "Raison",
    "Récompenses", "Révolution",
    "*", "*", "*", "*"
};

static const char *french_month_names[] = {
    "*", "Vendémiaire",
    "Brumaire", "Frimaire",
    "Nivôse", "Pluviôse",
    "Ventôse", "Germinal",
    "Floréal", "Prairial",
    "Messidor", "Thermidor",
    "Fructidor", "(Sansculottides)"
};

static const char *old_hindu_solar_month_names[] = {
    "*", "Meṣa", "Vṛṣabha",
    "Mithuna", "Karkaṭa", "Siṃha",
    "Kanyā", "Tulā", "Vṛścik‌‌‌a",
    "Dhanu", "Makara", "Kumbha",
    "Mīna"
};

static const char *old_hindu_lunar_month_names[] = {
    "*", "Chaitra", "Vaiśākha",
    "Jyaiṣṭha", "Āṣāḍha",
    "Śrāvaṇa", "Bhādrapada",
    "Āśvina", "Kārtika",
    "Mārgaśīrṣa", "Pauṣa",
    "Māgha", "Phālguna"
};

static const char *world_day_names[] = {
    "Sunday", "Monday", "Tuesday",
    "Wednesday", "Thursday",
    "Friday", "Saturday",
    "World", "Leap"
};

static const char *coptic_month_names[] = {
    "*", "Thout", "Paopi",
    "Hathor", "Koiak", "Tobi",
    "Meshir", "Paremhat","Parmouti",
    "Pashons", "Paoni", "Epip",
    "Mesori", "Pi Kogi Enavot"
};

static const char *ethiopian_month_names[] = {
    "*", "Mäskäräm", "Ṭəqəmt",
    "Ḫədar", "Taḫśaś'", "Ṭərr",
    "Yäkatit", "Mägabit",
    "Miyazya", "Gənbot", "Säne",
    "Ḥamle", "Nähase",
    "Ṗagʷəmen"
};

static const char *jalali_month_names[] = {
    "*", "Farvardin",
    "Ordibehesht ", "Khordad",
    "Tir", "Mordad", "Shahrivar",
    "Mehr", "Aban", "Azar",
    "Dey", "Bahman", "Esfand"
};

static const char *bahai_day_names[] = {
    "Jamál", "Kamál", "Fiḍál",
    "‘Idál", "Istijlál",
    "Istiqlál", "Jalál"
};

static const char *bahai_month_names[] = {
    "*", "Bahá", "Jalál",
    "Jamál", "‘Aẓamat", "Núr",
    "Raḥmat", "Kalimát", "Kamál",
    "Asmá’", "‘Izzat", "Mashíyyat",
    "‘Ilm", "Qudrat", "Qawl",
    "Masá’il", "Sharaf", "Sulṭán",
    "Mulk", "Ayyám-i-Há", "‘Alá’"
};

static const char *bahai_vahid_names[] = {
    "*", "Alif", "Bá’",
    "Ab", "Dál", "Báb",
    "Váv",  "Abad", "Jád",
    "Bahá'", "Ḥubb", "Bahháj",
    "Javáb", "Aḥad", "Vahháb",
    "Vidád", "Badí‘", "Bahí",
    "Abhá", "Váḥid"
};

static const char *indian_national_month_names[] = {
    "*", "Chaitra", "Vaishākha",
    "Jyaishtha", "Āshādha",
    "Shrāvana", "Bhādrapada",
    "Āshwin", "Kārtika",
    "Agrahayana", "Pausha",
    "Māgha", "Phālguna"
};

static const char *bengali_day_names[] = {
    "Rôbibar", "Sombar",
    "Mônggôlbar", "Budhbar",
    "Brihôspôtibar", "Shukrôbar",
    "Shônibar"
};

static const char *bengali_month_names[] = {
    "*", "Bôishakh", "Jyôishţhô",
    "Ashaŗh", "Shrabôn",
    "Bhadrô", "Ashbin",
    "Kartik", "Ogrôhayôn",
    "Poush", "Magh",
    "Falgun", "Chôitrô"
};

static const char *nanakshahi_month_names[] = {
    "*", "Chet",
    "Vaisakh", "Jeth",
    "Harh", "Sawan",
    "Bhadon", "Assu",
    "Katak", "Maghar",
    "Poh", "Magh",
    "Phagun"
};

static const char *kyureki_day_names[] = {
    "Senkachi", "Tomobiki",
    "Semmake", "Butsumetsu",
    "Taian", "Shakku"
};

static const char *kyureki_month_names[] = {
    "*", "Mutsuki", "Kisaragi",
    "Yayoi", "Uzuki", "Satsuki",
    "Minazuki", "Fuzuki",
    "Hazuki", "Nagatsuki",
    "Kannazuki", "Shimotsuki",
    "Shiwasu"
};

void
Init_calendrical(void)
{
    VALUE mod, cc_month_names;

    mod = rb_define_module("Calendrical");
    rb_define_module_function
	(mod, "gregorian_leap_year", calendrical_gregorian_leap_year, 1);
    rb_define_module_function
	(mod, "gregorian_leap_year?", calendrical_gregorian_leap_year, 1);
    rb_define_module_function
	(mod, "gregorian_last_day_of_month",
	 calendrical_gregorian_last_day_of_month, 2);
    rb_define_module_function
	(mod, "gregorian_to_rd", calendrical_gregorian_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_gregorian", calendrical_rd_to_gregorian, 1);
    rb_define_module_function
	(mod, "kday_on_or_before", calendrical_kday_on_or_before, 2);

    rb_define_module_function
	(mod, "iso_to_rd", calendrical_iso_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_iso", calendrical_rd_to_iso, 1);
    rb_define_module_function
	(mod, "week_date_to_rd", calendrical_iso_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_week_date",  calendrical_rd_to_iso, 1);

    rb_define_module_function
	(mod, "julian_leap_year", calendrical_julian_leap_year, 1);
    rb_define_module_function
	(mod, "julian_leap_year?", calendrical_julian_leap_year, 1);
    rb_define_module_function
	(mod, "julian_last_day_of_month",
	 calendrical_julian_last_day_of_month, 2);
    rb_define_module_function
	(mod, "julian_to_rd", calendrical_julian_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_julian", calendrical_rd_to_julian, 1);

    rb_define_module_function
	(mod, "islamic_leap_year", calendrical_islamic_leap_year, 1);
    rb_define_module_function
	(mod, "islamic_leap_year?", calendrical_islamic_leap_year, 1);
    rb_define_module_function
	(mod, "islamic_last_day_of_month",
	 calendrical_islamic_last_day_of_month, 2);
    rb_define_module_function
	(mod, "islamic_to_rd", calendrical_islamic_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_islamic", calendrical_rd_to_islamic, 1);

    rb_define_module_function
	(mod, "hebrew_leap_year", calendrical_hebrew_leap_year, 1);
    rb_define_module_function
	(mod, "hebrew_leap_year?", calendrical_hebrew_leap_year, 1);
    rb_define_module_function
	(mod, "hebrew_last_month_of_year",
	 calendrical_hebrew_last_month_of_year, 1);
    rb_define_module_function
	(mod, "hebrew_last_day_of_month",
	 calendrical_hebrew_last_day_of_month, 2);
    rb_define_module_function
	(mod, "hebrew_calendar_elapsed_days",
	 calendrical_hebrew_calendar_elapsed_days, 1);
    rb_define_module_function
	(mod, "hebrew_days_in_year", calendrical_hebrew_days_in_year, 1);
    rb_define_module_function
	(mod, "long_heshvan", calendrical_long_heshvan, 1);
    rb_define_module_function
	(mod, "long_heshvan?", calendrical_long_heshvan, 1);
    rb_define_module_function
	(mod, "short_kislev", calendrical_short_kislev, 1);
    rb_define_module_function
	(mod, "short_kislev?", calendrical_short_kislev, 1);
    rb_define_module_function
	(mod, "hebrew_to_rd", calendrical_hebrew_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_hebrew", calendrical_rd_to_hebrew, 1);

    rb_define_module_function
	(mod, "nth_kday", calendrical_nth_kday, 4);
    rb_define_module_function
	(mod, "nicaean_rule_easter", calendrical_nicaean_rule_easter, 1);
    rb_define_module_function
	(mod, "easter", calendrical_easter, 1);

    rb_define_module_function
	(mod, "mayan_long_count_to_rd",
	 calendrical_mayan_long_count_to_rd, 5);
    rb_define_module_function
	(mod, "rd_to_mayan_long_count",
	 calendrical_rd_to_mayan_long_count, 1);
    rb_define_module_function
	(mod, "rd_to_mayan_haab",
	 calendrical_rd_to_mayan_haab, 1);
    rb_define_module_function
	(mod, "mayan_haab_difference", calendrical_mayan_haab_difference, 4);
    rb_define_module_function
	(mod, "mayan_haab_on_or_before", calendrical_mayan_haab_on_or_before, 3);
    rb_define_module_function
	(mod, "rd_to_mayan_tzolkin",
	 calendrical_rd_to_mayan_tzolkin, 1);
    rb_define_module_function
	(mod, "mayan_tzolkin_difference",
	 calendrical_mayan_tzolkin_difference, 4);
    rb_define_module_function
	(mod, "mayan_tzolkin_on_or_before",
	 calendrical_mayan_tzolkin_on_or_before, 3);
    rb_define_module_function
	(mod, "mayan_haab_tzolkin_on_or_before",
	 calendrical_mayan_haab_tzolkin_on_or_before, 5);

    rb_define_module_function
	(mod, "french_leap_year", calendrical_french_leap_year, 1);
    rb_define_module_function
	(mod, "french_leap_year?", calendrical_french_leap_year, 1);
    rb_define_module_function
	(mod, "french_last_day_of_month",
	 calendrical_french_last_day_of_month, 2);
    rb_define_module_function
	(mod, "french_to_rd", calendrical_french_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_french", calendrical_rd_to_french, 1);

    rb_define_module_function
	(mod, "solar_longitude", calendrical_solar_longitude, 1);
    rb_define_module_function
	(mod, "zodiac", calendrical_zodiac, 1);
    rb_define_module_function
	(mod, "rd_to_old_hindu_solar",
	 calendrical_rd_to_old_hindu_solar, 1);
    rb_define_module_function
	(mod, "old_hindu_solar_to_rd",
	 calendrical_old_hindu_solar_to_rd, 3);
    rb_define_module_function
	(mod, "lunar_longitude", calendrical_lunar_longitude, 1);
    rb_define_module_function
	(mod, "lunar_phase", calendrical_lunar_phase, 1);
    rb_define_module_function
	(mod, "new_moon", calendrical_new_moon, 1);
    rb_define_module_function
	(mod, "rd_to_old_hindu_lunar",
	 calendrical_rd_to_old_hindu_lunar, 1);
    rb_define_module_function
	(mod, "old_hindu_lunar_precedes",
	 calendrical_old_hindu_lunar_precedes, 8);
    rb_define_module_function
	(mod, "old_hindu_lunar_to_rd",
	 calendrical_old_hindu_lunar_to_rd, 4);

    rb_define_module_function
	(mod, "world_leap_year", calendrical_world_leap_year, 1);
    rb_define_module_function
	(mod, "world_leap_year?", calendrical_world_leap_year, 1);
    rb_define_module_function
	(mod, "world_last_day_of_month", calendrical_world_last_day_of_month, 2);
    rb_define_module_function
	(mod, "world_to_rd", calendrical_world_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_world", calendrical_rd_to_world, 1);
    rb_define_module_function
	(mod, "rd_to_world_day_of_week",
	 calendrical_rd_to_world_day_of_week, 1);

    rb_define_module_function
	(mod, "coptic_leap_year", calendrical_coptic_leap_year, 1);
    rb_define_module_function
	(mod, "coptic_leap_year?", calendrical_coptic_leap_year, 1);
    rb_define_module_function
	(mod, "coptic_last_day_of_month",
	 calendrical_coptic_last_day_of_month, 2);
    rb_define_module_function
	(mod, "coptic_to_rd", calendrical_coptic_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_coptic", calendrical_rd_to_coptic, 1);

    rb_define_module_function
	(mod, "ethiopian_leap_year", calendrical_ethiopian_leap_year, 1);
    rb_define_module_function
	(mod, "ethiopian_leap_year?", calendrical_ethiopian_leap_year, 1);
    rb_define_module_function
	(mod, "ethiopian_last_day_of_month",
	 calendrical_ethiopian_last_day_of_month, 2);
    rb_define_module_function
	(mod, "ethiopian_to_rd", calendrical_ethiopian_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_ethiopian", calendrical_rd_to_ethiopian, 1);

    rb_define_module_function
	(mod, "jalali_leap_year", calendrical_jalali_leap_year, 1);
    rb_define_module_function
	(mod, "jalali_leap_year?", calendrical_jalali_leap_year, 1);
    rb_define_module_function
	(mod, "jalali_last_day_of_month",
	 calendrical_jalali_last_day_of_month, 2);
    rb_define_module_function
	(mod, "jalali_to_rd", calendrical_jalali_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_jalali", calendrical_rd_to_jalali, 1);

    rb_define_module_function
	(mod, "bahai_leap_year", calendrical_bahai_leap_year, 1);
    rb_define_module_function
	(mod, "bahai_leap_year?", calendrical_bahai_leap_year, 1);
    rb_define_module_function
	(mod, "bahai_last_day_of_month",
	 calendrical_bahai_last_day_of_month, 2);
    rb_define_module_function
	(mod, "bahai_to_rd", calendrical_bahai_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_bahai", calendrical_rd_to_bahai, 1);
    rb_define_module_function
	(mod, "bahai_year_to_vahid",
	 calendrical_bahai_year_to_vahid, 1);
    rb_define_module_function
	(mod, "bahai_vahid_to_year",
	 calendrical_bahai_vahid_to_year, 3);

    rb_define_module_function
	(mod, "indian_national_leap_year", calendrical_indian_national_leap_year, 1);
    rb_define_module_function
	(mod, "indian_national_leap_year?", calendrical_indian_national_leap_year, 1);
    rb_define_module_function
	(mod, "indian_national_last_day_of_month",
	 calendrical_indian_national_last_day_of_month, 2);
    rb_define_module_function
	(mod, "indian_national_to_rd", calendrical_indian_national_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_indian_national", calendrical_rd_to_indian_national, 1);

    rb_define_module_function
	(mod, "bengali_leap_year", calendrical_bengali_leap_year, 1);
    rb_define_module_function
	(mod, "bengali_leap_year?", calendrical_bengali_leap_year, 1);
    rb_define_module_function
	(mod, "bengali_last_day_of_month",
	 calendrical_bengali_last_day_of_month, 2);
    rb_define_module_function
	(mod, "bengali_to_rd", calendrical_bengali_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_bengali", calendrical_rd_to_bengali, 1);

    rb_define_module_function
	(mod, "nanakshahi_leap_year", calendrical_nanakshahi_leap_year, 1);
    rb_define_module_function
	(mod, "nanakshahi_leap_year?", calendrical_nanakshahi_leap_year, 1);
    rb_define_module_function
	(mod, "nanakshahi_last_day_of_month",
	 calendrical_nanakshahi_last_day_of_month, 2);
    rb_define_module_function
	(mod, "nanakshahi_to_rd", calendrical_nanakshahi_to_rd, 3);
    rb_define_module_function
	(mod, "rd_to_nanakshahi", calendrical_rd_to_nanakshahi, 1);

    rb_define_module_function
	(mod, "ordinal_to_rd", calendrical_ordinal_to_rd, 2);
    rb_define_module_function
	(mod, "rd_to_ordinal", calendrical_rd_to_ordinal, 1);

    rb_define_module_function
	(mod, "jd_to_rd", calendrical_jd_to_rd, 1);
    rb_define_module_function
	(mod, "rd_to_jd", calendrical_rd_to_jd, 1);

    rb_define_module_function
	(mod, "mjd_to_rd", calendrical_mjd_to_rd, 1);
    rb_define_module_function
	(mod, "rd_to_mjd", calendrical_rd_to_mjd, 1);

    rb_define_module_function
	(mod, "mjd_to_jd", calendrical_mjd_to_jd, 1);
    rb_define_module_function
	(mod, "jd_to_mjd", calendrical_jd_to_mjd, 1);

    rb_define_module_function
	(mod, "ld_to_rd", calendrical_ld_to_rd, 1);
    rb_define_module_function
	(mod, "rd_to_ld", calendrical_rd_to_ld, 1);

    rb_define_module_function
	(mod, "rd_to_day_of_week", calendrical_rd_to_day_of_week, 1);

    rb_define_module_function
	(mod, "kyureki_to_rd", calendrical_kyureki_to_rd, 4);
    rb_define_module_function
	(mod, "rd_to_kyureki", calendrical_rd_to_kyureki, 1);
    rb_define_module_function
	(mod, "rd_to_kyureki_day_of_week",
	 calendrical_rd_to_kyureki_day_of_week, 1);

    rb_define_const
	(mod, "VERSION",
	 rb_obj_freeze(rb_str_new2("$Revision: 1.17 $")));

#define const_array(a) mk_ary_of_str(sizeof a / sizeof a[0], a)
#define def_const(n, o) rb_define_const(mod, n, o)
#define def_const_array(n, a) def_const(n, const_array(a))

    def_const_array("DAY_NAMES", day_names);
    cc_month_names = const_array(month_names);
    def_const("MONTH_NAMES", cc_month_names);

    def_const_array("ISLAMIC_DAY_NAMES", islamic_day_names);
    def_const_array("ISLAMIC_MONTH_NAMES", islamic_month_names);
    def_const_array("HEBREW_DAY_NAMES", hebrew_day_names);
    def_const("HEBREW_MONTH_NAMES",
	      rb_obj_freeze(rb_ary_new3(2,
					const_array(hebrew_common_year_month_names),
					const_array(hebrew_leap_year_month_names))));

    def_const_array("MAYAN_HAAB_MONTH_NAMES", mayan_haab_month_names);
    def_const_array("MAYAN_TZOLKIN_MONTH_NAMES", mayan_tzolkin_month_names);

    def_const_array("FRENCH_DAY_NAMES", french_day_names);
    def_const_array("FRENCH_SANSCULOTTIDES_NAMES", french_sansculottides_names);
    def_const_array("FRENCH_MONTH_NAMES", french_month_names);

    def_const_array("OLD_HINDU_SOLAR_MONTH_NAMES", old_hindu_solar_month_names);
    def_const_array("OLD_HINDU_LUNAR_MONTH_NAMES", old_hindu_lunar_month_names);

    def_const_array("WORLD_DAY_NAMES", world_day_names);

    def_const_array("COPTIC_MONTH_NAMES", coptic_month_names);
    def_const_array("ETHIOPIAN_MONTH_NAMES", ethiopian_month_names);
    def_const_array("JALALI_MONTH_NAMES", jalali_month_names);

    def_const_array("BAHAI_DAY_NAMES", bahai_day_names);
    def_const_array("BAHAI_MONTH_NAMES", bahai_month_names);
    def_const_array("BAHAI_VAHID_NAMES", bahai_vahid_names);

    def_const_array("INDIAN_NATIONAL_MONTH_NAMES", indian_national_month_names);

    def_const_array("BENGALI_DAY_NAMES", bengali_day_names);
    def_const_array("BENGALI_MONTH_NAMES", bengali_month_names);

    def_const_array("NANAKSHAHI_MONTH_NAMES", nanakshahi_month_names);

    def_const_array("KYUREKI_DAY_NAMES", kyureki_day_names);
    def_const_array("KYUREKI_MONTH_NAMES", kyureki_month_names);

    def_const("GREGORIAN_MONTH_NAMES", cc_month_names);
    def_const("JULIAN_MONTH_NAMES", cc_month_names);
    def_const("WORLD_MONTH_NAMES", cc_month_names);
}

/*
Local variables:
c-file-style: "ruby"
End:
*/
